@@ -5,6 +5,7 @@
 from ctypes import c_uint8, c_uint16, c_uint32, c_uint64, Structure
 import socket
 import struct
+from nxc.helpers.misc import CATEGORY
 from nxc.logger import nxc_logger
 
 
@@ -31,19 +32,19 @@ class SmbHeader(Structure):
     ]
 
     def __init__(self, buffer):
-        nxc_logger.debug("server_component : %04x" % self.server_component)
-        nxc_logger.debug("smb_command      : %01x" % self.smb_command)
-        nxc_logger.debug("error_class      : %01x" % self.error_class)
-        nxc_logger.debug("error_code       : %02x" % self.error_code)
-        nxc_logger.debug("flags            : %01x" % self.flags)
-        nxc_logger.debug("flags2           : %02x" % self.flags2)
-        nxc_logger.debug("process_id_high  : %02x" % self.process_id_high)
-        nxc_logger.debug("signature        : %08x" % self.signature)
-        nxc_logger.debug("reserved2        : %02x" % self.reserved2)
-        nxc_logger.debug("tree_id          : %02x" % self.tree_id)
-        nxc_logger.debug("process_id       : %02x" % self.process_id)
-        nxc_logger.debug("user_id          : %02x" % self.user_id)
-        nxc_logger.debug("multiplex_id     : %02x" % self.multiplex_id)
+        nxc_logger.debug(f"server_component : {self.server_component:04x}")
+        nxc_logger.debug(f"smb_command      : {self.smb_command:01x}")
+        nxc_logger.debug(f"error_class      : {self.error_class:01x}")
+        nxc_logger.debug(f"error_code       : {self.error_code:02x}")
+        nxc_logger.debug(f"flags            : {self.flags:01x}")
+        nxc_logger.debug(f"flags2           : {self.flags2:02x}")
+        nxc_logger.debug(f"process_id_high  : {self.process_id_high:02x}")
+        nxc_logger.debug(f"signature        : {self.signature:08x}")
+        nxc_logger.debug(f"reserved2        : {self.reserved2:02x}")
+        nxc_logger.debug(f"tree_id          : {self.tree_id:02x}")
+        nxc_logger.debug(f"process_id       : {self.process_id:02x}")
+        nxc_logger.debug(f"user_id          : {self.user_id:02x}")
+        nxc_logger.debug(f"multiplex_id     : {self.multiplex_id:02x}")
 
     def __new__(self, buffer=None):
         nxc_logger.debug(f"Creating SMB_HEADER object from buffer: {buffer}")
@@ -54,8 +55,7 @@ class NXCModule:
     name = "ms17-010"
     description = "MS17-010 - EternalBlue - NOT TESTED OUTSIDE LAB ENVIRONMENT"
     supported_protocols = ["smb"]
-    opsec_safe = True
-    multiple_hosts = True
+    category = CATEGORY.ENUMERATION
 
     def options(self, context, module_options):
         """ """
@@ -72,7 +72,6 @@ class NXCModule:
             if str(e) == "Buffer size too small (0 instead of at least 32 bytes)":
                 context.log.debug("Buffer size too small, which means the response was not the expected size")
 
-
     def generate_smb_proto_payload(self, *protos):
         """
         Flattens a nested list and merges all bytes objects into a single bytes object.
@@ -98,7 +97,6 @@ class NXCModule:
         self.logger.debug(f"Packed proto data: {hex_data}")
         return hex_data
 
-
     def calculate_doublepulsar_xor_key(self, s):
         """
         Calculate Doublepulsar Xor Key.
@@ -115,8 +113,6 @@ class NXCModule:
         x = (2 * s ^ (((s & 0xff00 | (s << 16)) << 8) | (((s >> 16) | s & 0xff0000) >> 8)))
         return x & 0xffffffff  # truncate to 32 bits
 
-
-
     def negotiate_proto_request(self):
         """Generate a negotiate_proto_request packet."""
         self.logger.debug("generate negotiate proto request")
@@ -160,7 +156,6 @@ class NXCModule:
         # Return the generated SMB protocol payload
         return self.generate_smb_proto_payload(netbios, smb_header, negotiate_proto_request)
 
-
     def session_setup_andx_request(self):
         """Generate session setup andx request."""
         self.logger.debug("generate session setup andx request"
@@ -210,7 +205,6 @@ class NXCModule:
 
         return self.generate_smb_proto_payload(netbios, smb_header, session_setup_andx_request)
 
-
     def tree_connect_andx_request(self, ip, userid):
         """Generate tree connect andx request.
 
@@ -279,7 +273,6 @@ class NXCModule:
         # Generate the final SMB protocol payload
         return self.generate_smb_proto_payload(netbios, smb_header, tree_connect_andx_request)
 
-
     def peeknamedpipe_request(self, treeid, processid, userid, multiplex_id):
         """
         Generate tran2 request.
@@ -345,7 +338,6 @@ class NXCModule:
 
         return self.generate_smb_proto_payload(netbios, smb_header, tran_request)
 
-
     def trans2_request(self, treeid, processid, userid, multiplex_id):
         """Generate trans2 request.
 
@@ -409,7 +401,6 @@ class NXCModule:
 
         return self.generate_smb_proto_payload(netbios, smb_header, trans2_request)
 
-
     def check(self, ip, port=445):
         """Check if MS17_010 SMB Vulnerability exists.
 
